@@ -44,7 +44,8 @@ public:
 
     char* dirName = nullptr;            //base resource directory
     bool copy = false;                  //"content" is owned by this loader
-    bool overriden = false;             //overridden properties with slots.
+    bool overriden = false;             //overridden properties with slots
+    bool rebuild = false;               //require building the lottie scene
 
     LottieLoader();
     ~LottieLoader();
