@@ -344,18 +344,15 @@ void LottieGroup::prepare(LottieObject::Type type)
 
 LottieLayer::~LottieLayer()
 {
-    if (refId) {
-        //No need to free assets children because the Composition owns them.
-        children.clear();
-        free(refId);
-    }
+    //No need to free assets children because the Composition owns them.
+    if (rid) children.clear();
 
     for (auto m = masks.begin(); m < masks.end(); ++m) {
         delete(*m);
     }
 
-    delete(matte.target);
     delete(transform);
+    free(name);
 }
 
 void LottieLayer::prepare()
