@@ -42,7 +42,7 @@ TEST_CASE("Lottie Slot", "[tvgLottie]")
     REQUIRE(animation);
 
     auto picture = animation->picture();
-    REQUIRE(picture->identifier == Picture::identifier);
+    REQUIRE(picture->identifier() == Picture::identifier());
 
     const char* slotJson = R"({"gradient_fill":{"p":{"a":0,"k":[0,0.1,0.1,0.2,1,1,0.1,0.2,0.1,1]}}})";
 
@@ -70,4 +70,38 @@ TEST_CASE("Lottie Slot", "[tvgLottie]")
     REQUIRE(Initializer::term(CanvasEngine::Sw) == Result::Success);
 }
 
+TEST_CASE("Lottie Marker", "[tvgLottie]")
+{
+    REQUIRE(Initializer::init(tvg::CanvasEngine::Sw, 0) == Result::Success);
+
+    auto animation = LottieAnimation::gen();
+    REQUIRE(animation);
+
+    auto picture = animation->picture();
+    REQUIRE(picture->identifier() == Picture::identifier());
+
+    //Set marker name before loaded
+    REQUIRE(animation->segment("sectionC") == Result::InsufficientCondition);
+
+    //Animation load
+    REQUIRE(picture->load(TEST_DIR"/lottiemarker.json") == Result::Success);
+
+    //Set marker
+    REQUIRE(animation->segment("sectionA") == Result::Success);
+
+    //Set marker by invalid name
+    REQUIRE(animation->segment("") == Result::InvalidArguments);
+
+    //Get marker count
+    REQUIRE(animation->markersCnt() == 3);
+
+    //Get marker name by index
+    REQUIRE(!strcmp(animation->marker(1), "sectionB"));
+
+    //Get marker name by invalid index
+    REQUIRE(animation->marker(-1) == nullptr);
+
+    REQUIRE(Initializer::term(tvg::CanvasEngine::Sw) == Result::Success);
+}
+
 #endif
\ No newline at end of file
